#!/usr/bin/env python3
# coding: utf-8
# Until PEP 517 is more widespread, we use this stub to provide backward compatibility
# install using, for example, `python3 setup.py install`
import setuptools
setuptools.setup()
